@@ -55,90 +55,90 @@ sub _twist {
     my $request = shift;
 
     my $list    = $request->{context};
-    my $which   = $list->{'name'};
-    my $robot   = $list->{'domain'};
     my $sender  = $request->{sender};
     my $email   = $request->{email};
     my $comment = $request->{gecos};
+    my $role    = $request->{role} || 'member';
     my $ca      = $request->{custom_attribute};
 
+    die 'bug in logic. Ask developer'
+        unless grep { $role eq $_ } qw(member owner editor);
+
     $language->set_lang($list->{'admin'}{'lang'});
 
     unless (Sympa::Tools::Text::valid_email($email)) {
         $self->add_stash($request, 'user', 'incorrect_email',
             {'email' => $email});
         $log->syslog('err',
-            'ADD command rejected; incorrect email "%s"', $email);
+            'request "add" rejected; incorrect email "%s"', $email);
         return undef;
     }
 
-    if (Sympa::Tools::Domains::is_blocklisted($email)) {
-        $self->add_stash($request, 'user', 'blocklisted_domain',
-            {'email' => $email});
-        $log->syslog('err',
-            'ADD command rejected; blocklisted domain for "%s"', $email);
-        return undef;
-    }
-
-    if ($list->is_list_member($email)) {
-        $self->add_stash($request, 'user', 'already_subscriber',
-            {'email' => $email, 'listname' => $list->{'name'}});
-        $log->syslog('err',
-            'ADD command rejected; user "%s" already member of list "%s"',
-            $email, $which);
-        return undef;
-    }
-
-    unless ($request->{force}) {
-        # If a list is not 'open' and allow_subscribe_if_pending has been set
-        # to 'off' returns undef.
-        unless (
-            $list->{'admin'}{'status'} eq 'open'
-            or Conf::get_robot_conf($list->{'domain'},
-                'allow_subscribe_if_pending') eq 'on'
-        ) {
+    my @stash;
+    if ($role eq 'member') {
+        unless ($request->{force} or $list->is_subscription_allowed) {
+            $log->syslog('info', 'List %s not open', $list);
             $self->add_stash($request, 'user', 'list_not_open',
                 {'status' => $list->{'admin'}{'status'}});
-            $log->syslog('info', 'List %s not open', $list);
+            $self->{finish} = 1;
             return undef;
         }
+        if (Sympa::Tools::Domains::is_blocklisted($email)) {
+            $self->add_stash($request, 'user', 'blocklisted_domain',
+                {'email' => $email});
+            $log->syslog('err',
+                'request "add" rejected; blocklisted domain for "%s"',
+                $email);
+            return undef;
+        }
+
+        $list->add_list_member(
+            {email => $email, gecos => $comment, custom_attribute => $ca},
+            stash => \@stash);
+    } else {
+        $list->add_list_admin(
+            $role,
+            {email => $email, gecos => $comment},
+            stash => \@stash
+        );
     }
-
-    my $u;
-    my $defaults = $list->get_default_user_options();
-    %{$u} = %{$defaults};
-    $u->{'email'} = $email;
-    $u->{'gecos'} = $comment;
-    $u->{'date'}  = $u->{'update_date'} = time;
-    $u->{custom_attribute} = $ca if $ca;
-
-    $list->add_list_member($u);
-    if (defined $list->{'add_outcome'}{'errors'}) {
-        if (defined $list->{'add_outcome'}{'errors'}
-            {'max_list_members_exceeded'}) {
-            $self->add_stash($request, 'user', 'max_list_members_exceeded',
-                {max_list_members => $list->{'admin'}{'max_list_members'}});
-        } else {
-            my $error =
-                sprintf 'Unable to add user %s in list %s : %s',
-                $u, $list->get_id,
-                $list->{'add_outcome'}{'errors'}{'error_message'};
+    foreach my $report (@stash) {
+        $self->add_stash($request, @$report);
+        if ($report->[0] eq 'intern') {
             Sympa::send_notify_to_listmaster(
                 $list,
                 'mail_intern_error',
-                {   error  => $error,
+                {   error  => $report->[1],      #FIXME: Update listmaster tt2
                     who    => $sender,
                     action => 'Command process',
                 }
             );
-            $self->add_stash($request, 'intern');
         }
-        return undef;
     }
+    return undef if grep { $_->[0] eq 'user' or $_->[0] eq 'intern' } @stash;
+
+    if ($role eq 'member') {
+        _report_member($self, $request);
+    } else {
+        _report_user($self, $request);
+    }
+
+    return 1;
+}
+
+sub _report_member {
+    my $self    = shift;
+    my $request = shift;
+
+    my $list    = $request->{context};
+    my $sender  = $request->{sender};
+    my $email   = $request->{email};
+    my $comment = $request->{gecos};
 
     $self->add_stash($request, 'notice', 'now_subscriber',
         {'email' => $email, listname => $list->{'name'}});
 
+    # FIXME: Required?
     my $user = Sympa::User->new($email);
     $user->lang($list->{'admin'}{'lang'}) unless $user->lang;
     $user->save;
@@ -157,7 +157,7 @@ sub _twist {
     $log->syslog(
         'info',
         'ADD %s %s from %s accepted (%.2f seconds, %d subscribers)',
-        $which,
+        $list->{'name'},
         $email,
         $sender,
         Time::HiRes::time() - $self->{start_time},
@@ -173,7 +173,34 @@ sub _twist {
             }
         );
     }
-    return 1;
+}
+
+sub _report_user {
+    my $self    = shift;
+    my $request = shift;
+
+    my $list   = $request->{context};
+    my $role   = $request->{role};
+    my $email  = $request->{email};
+    my $sender = $request->{sender};
+
+    # Notify the new list owner/editor
+    unless ($request->{quiet}) {
+        Sympa::send_notify_to_user($list, 'added_as_listadmin', $email,
+            {admin_type => $role, delegator => $sender});
+        $self->add_stash($request, 'notice', 'user_notified',
+            {'notified_user' => $email});
+    }
+
+    $log->syslog(
+        'info',
+        'request "add" %s %s to %s from %s accepted (%.2f seconds)',
+        $role,
+        $email,
+        $list,
+        $sender,
+        Time::HiRes::time() - $self->{start_time}
+    );
 }
 
 1;
@@ -193,6 +220,11 @@ quiet add has been chosen (which requires the
 quiet_subscription setting to be "optional") or forced (which
 requires the quiet_subscription setting to be "on").
 
+B<Note>:
+The autharization secenario C<add.*> is applicable only when the {role}
+attribute is C<'member'> (default).
+In the other cases the scenario processing should be skipped.
+
 =head2 Attributes
 
 See also L<Sympa::Request/"Attributes">.
@@ -215,6 +247,14 @@ users will be added even if the list is closed.
 I<Optional>.
 Display name of the user to be added.
 
+=item {role}
+
+I<Optional>.
+Role of the user to be added: C<'member'>, C<'owner'> or C<'editor'>.
+Default value is C<'member'>.
+
+This attribute was introduced on Sympa 6.2.67b.2.
+
 =item {quiet}
 
 I<Optional>.
